#pragma once
#include "uart/ProtocolSender.h"
/*
*此文件由GUI工具生成
*文件功能：用于处理用户的逻辑相应代码
*功能说明：
*========================onButtonClick_XXXX
当页面中的按键按下后系统会调用对应的函数，XXX代表GUI工具里面的[ID值]名称，
如Button1,当返回值为false的时候系统将不再处理这个按键，返回true的时候系统将会继续处理此按键。比如SYS_BACK.
*========================onSlideWindowItemClick_XXXX(int index) 
当页面中存在滑动窗口并且用户点击了滑动窗口的图标后系统会调用此函数,XXX代表GUI工具里面的[ID值]名称，
如slideWindow1;index 代表按下图标的偏移值
*========================onSeekBarChange_XXXX(int progress) 
当页面中存在滑动条并且用户改变了进度后系统会调用此函数,XXX代表GUI工具里面的[ID值]名称，
如SeekBar1;progress 代表当前的进度值
*========================ogetListItemCount_XXXX() 
当页面中存在滑动列表的时候，更新的时候系统会调用此接口获取列表的总数目,XXX代表GUI工具里面的[ID值]名称，
如List1;返回值为当前列表的总条数
*========================oobtainListItemData_XXXX(ZKListView::ZKListItem *pListItem, int index)
 当页面中存在滑动列表的时候，更新的时候系统会调用此接口获取列表当前条目下的内容信息,XXX代表GUI工具里面的[ID值]名称，
如List1;pListItem 是贴图中的单条目对象，index是列表总目的偏移量。具体见函数说明
*========================常用接口===============
*LOGD(...)  打印调试信息的接口
*mTextXXXPtr->setText("****") 在控件TextXXX上显示文字****
*mButton1Ptr->setSelected(true); 将控件mButton1设置为选中模式，图片会切换成选中图片，按钮文字会切换为选中后的颜色
*mSeekBarPtr->setProgress(12) 在控件mSeekBar上将进度调整到12
*mListView1Ptr->refreshListView() 让mListView1 重新刷新，当列表数据变化后调用
*mDashbroadView1Ptr->setTargetAngle(120) 在控件mDashbroadView1上指针显示角度调整到120度
*
* 在Eclipse编辑器中  使用 “alt + /”  快捷键可以打开智能提示
*/

#include "net/NetManager.h"
#include "server/http_server.h"

#define HTTP_SERVER_PORT  80

srv::HttpServer httpServer;

// percent 0~100
static void UISetProgress(int percent) {
  //根据百分比计算控件宽度及位置，实现进度条效果
  double ratio = percent / 100.0;
  LayoutPosition pos_progress = mTextViewProgressBackgroundPtr->getPosition();
  int width = ratio * pos_progress.mWidth;
  pos_progress.mWidth = width;
  mTextViewProgressForegroundPtr->setPosition(pos_progress);

  LayoutPosition pos_number = mTextViewProgressPercentPtr->getPosition();
  pos_number.mLeft = pos_progress.mLeft + pos_progress.mWidth + 5;
  if (percent <= 0) {
    pos_number.mWidth = 0;
  } else {
    pos_number.mWidth = 60;
  }
  mTextViewProgressPercentPtr->setPosition(pos_number);
  mTextViewProgressPercentPtr->setText(srv::format("%d%%", percent));

}

static void UISetProgress(uint64_t value, uint64_t max_value) {
  if (max_value == 0) {
    UISetProgress(0);
    return;
  }

  static int last_percent = 0;

  double ratio = value * 1.0 / max_value;
  int percent = (int)(ratio * 100);
  if (percent != last_percent) {  //有变化才更新
    UISetProgress(last_percent = percent);
  }
}

static void UploadFilenameListener(const std::string &name) {
  mTextViewFilenamePtr->setText(name);
}

static void UploadProgressListener(uint64_t uploaded_bytes, uint64_t all_bytes) {
  UISetProgress(uploaded_bytes, all_bytes);
}

static std::string GetURL(const char* ip, int port) {
  if (ip == NULL) {
    return "";
  }
  if (port == 80) {
    return srv::format("http://%s", ip).c_str();
  }
  return srv::format("http://%s:%d", ip, port).c_str();
}

static void UpdateQRCode() {
  //检查网络是否连接，得到IP，更新二维码
  const char *ip = NETMANAGER->getEthernetManager()->getIp();
  if (ip != NULL) {
    mQRCode1Ptr->loadQRCode(GetURL(ip, HTTP_SERVER_PORT).c_str());
  } else {
    mQRCode1Ptr->loadQRCode("未连接网络");
  }
}

/**
 * 注册定时器
 * 填充数组用于注册定时器
 * 注意：id不能重复
 */
static S_ACTIVITY_TIMEER REGISTER_ACTIVITY_TIMER_TAB[] = {
	//{0,  6000}, //定时器id=0, 时间间隔6秒
	{1,  3000},
};


/**
 * 当界面构造时触发
 */
static void onUI_init(){

  //设置上传文件名监听
  httpServer.set_upload_filename_listener(UploadFilenameListener);
  //设置上传进度监听
  httpServer.set_upload_progress_listener(UploadProgressListener);
  httpServer.RunAsync(HTTP_SERVER_PORT);
  UISetProgress(0);
  mTextViewFilenamePtr->setText("");

  const std::string tips = srv::format(
      "1. 连网，使手机与设备处于同一网络。\n"\
      "2. 必须插入TF卡，文件将保存在TF中。\n"\
      "3. 扫描二维码上传文件，\n"\
      "     或者访问 %s", GetURL(NETMANAGER->getEthernetManager()->getIp(), HTTP_SERVER_PORT).c_str());

  mTextViewTipsPtr->setText(tips);

  UpdateQRCode();
}

/**
 * 当切换到该界面时触发
 */
static void onUI_intent(const Intent *intentPtr) {
    if (intentPtr != NULL) {
        //TODO
    }
}

/*
 * 当界面显示时触发
 */
static void onUI_show() {

}

/*
 * 当界面隐藏时触发
 */
static void onUI_hide() {

}

/*
 * 当界面完全退出时触发
 */
static void onUI_quit() {

}

/**
 * 串口数据回调接口
 */
static void onProtocolDataUpdate(const SProtocolData &data) {

}

/**
 * 定时器触发函数
 * 不建议在此函数中写耗时操作，否则将影响UI刷新
 * 参数： id
 *         当前所触发定时器的id，与注册时的id相同
 * 返回值: true
 *             继续运行当前定时器
 *         false
 *             停止运行当前定时器
 */
static bool onUI_Timer(int id){
	switch (id) {
	case 1: {
	  UpdateQRCode();
	}
	  break;
		default:
			break;
	}
    return true;
}

/**
 * 有新的触摸事件时触发
 * 参数：ev
 *         新的触摸事件
 * 返回值：true
 *            表示该触摸事件在此被拦截，系统不再将此触摸事件传递到控件上
 *         false
 *            触摸事件将继续传递到控件上
 */
static bool onmainActivityTouchEvent(const MotionEvent &ev) {
    switch (ev.mActionStatus) {
		case MotionEvent::E_ACTION_DOWN://触摸按下
			//LOGD("时刻 = %ld 坐标  x = %d, y = %d", ev.mEventTime, ev.mX, ev.mY);
			break;
		case MotionEvent::E_ACTION_MOVE://触摸滑动
			break;
		case MotionEvent::E_ACTION_UP:  //触摸抬起
			break;
		default:
			break;
	}
	return false;
}
static bool onButtonClick_Button1(ZKButton *pButton) {
  EASYUICONTEXT->openActivity("NetSettingActivity");
  return false;
}

